@@ -12,10 +12,13 @@
 # for more details.
 
 
-import sys
+from inspect import getargspec
 import os
+import sys
 import traceback
+from pkg_resources import parse_version
 
+from IPython.terminal.ipapp import load_default_config
 from contextlib import contextmanager
 
 try:
@@ -26,38 +29,71 @@ except ImportError:
 
 import IPython
 
-if IPython.__version__ > '0.10.2':
+
+def import_module(possible_modules, needed_module):
+    """Make it more resilient to different versions of IPython and try to
+    find a module."""
+    count = len(possible_modules)
+    for module in possible_modules:
+        try:
+            return __import__(module, fromlist=[needed_module])
+        except ImportError:
+            count -= 1
+            if count == 0:
+                raise
+
+if parse_version(IPython.__version__) > parse_version('0.10.2'):
     from IPython.core.debugger import Pdb, BdbQuit_excepthook
+
+    possible_modules = ['IPython.terminal.ipapp',           # Newer IPython
+                        'IPython.frontend.terminal.ipapp']  # Older IPython
+
+    app = import_module(possible_modules, "TerminalIPythonApp")
+    TerminalIPythonApp = app.TerminalIPythonApp
+
+    possible_modules = ['IPython.terminal.embed',           # Newer IPython
+                        'IPython.frontend.terminal.embed']  # Older IPython
+    embed = import_module(possible_modules, "InteractiveShellEmbed")
+    InteractiveShellEmbed = embed.InteractiveShellEmbed
     try:
         get_ipython
     except NameError:
-        # Make it more resilient to different versions of IPython and try to
-        # find a module.
-        possible_modules = ['IPython.terminal.embed',           # Newer IPython
-                            'IPython.frontend.terminal.embed']  # Older IPython
-
-        count = len(possible_modules)
-        for module in possible_modules:
-            try:
-                embed = __import__(module, fromlist=["InteractiveShellEmbed"])
-                InteractiveShellEmbed = embed.InteractiveShellEmbed
-            except ImportError:
-                count -= 1
-                if count == 0:
-                    raise
-            else:
-                break
-
-        ipshell = InteractiveShellEmbed()
-        def_colors = ipshell.colors
+        # Build a terminal app in order to force ipython to load the
+        # configuration
+        ipapp = TerminalIPythonApp()
+        # Avoid output (banner, prints)
+        ipapp.interact = False
+        ipapp.initialize()
+        def_colors = ipapp.shell.colors
     else:
-        def_colors = get_ipython.__self__.colors
+        # If an instance of IPython is already running try to get an instance
+        # of the application. If there is no TerminalIPythonApp instanciated
+        # the instance method will create a new one without loading the config.
+        # i.e: if we are in an embed instance we do not want to load the config.
+        ipapp = TerminalIPythonApp.instance()
+        shell = get_ipython()
+        def_colors = shell.colors
+
+        # Detect if embed shell or not and display a message
+        if isinstance(shell, InteractiveShellEmbed):
+            shell.write_err(
+                "\nYou are currently into an embedded ipython shell,\n"
+                "the configuration will not be loaded.\n\n"
+            )
+
+
+
+    def_exec_lines = [line + '\n' for line in ipapp.exec_lines]
 
     from IPython.utils import io
 
-    def update_stdout():
-        # setup stdout to ensure output is available with nose
-        io.stdout = sys.stdout = sys.__stdout__
+    if 'nose' in list(sys.modules.keys()):
+        def update_stdout():
+            # setup stdout to ensure output is available with nose
+            io.stdout = sys.stdout = sys.__stdout__
+    else:
+        def update_stdout():
+            pass
 else:
     from IPython.Debugger import Pdb, BdbQuit_excepthook
     from IPython.Shell import IPShell
@@ -68,13 +104,25 @@ else:
         IPShell(argv=[''])
         ip = ipapi.get()
     def_colors = ip.options.colors
+    def_exec_lines = []
 
     from IPython.Shell import Term
 
-    def update_stdout():
-        # setup stdout to ensure output is available with nose
-        Term.cout = sys.stdout = sys.__stdout__
+    if 'nose' in list(sys.modules.keys()):
+        def update_stdout():
+            # setup stdout to ensure output is available with nose
+            Term.cout = sys.stdout = sys.__stdout__
+    else:
+        def update_stdout():
+            pass
 
+def _init_pdb(context=3):
+    if 'context' in getargspec(Pdb.__init__)[0]:
+        p = Pdb(def_colors, context=context)
+    else:
+        p = Pdb(def_colors)
+    p.rcLines += def_exec_lines
+    return p
 
 def wrap_sys_excepthook():
     # make sure we wrap it only once or we would end up with a cycle
@@ -84,24 +132,22 @@ def wrap_sys_excepthook():
         sys.excepthook = BdbQuit_excepthook
 
 
-def set_trace(frame=None):
+def set_trace(frame=None, context=3):
     update_stdout()
     wrap_sys_excepthook()
     if frame is None:
         frame = sys._getframe().f_back
-    Pdb(def_colors).set_trace(frame)
+    _init_pdb(context).set_trace(frame)
 
 
 def post_mortem(tb):
     update_stdout()
     wrap_sys_excepthook()
-    p = Pdb(def_colors)
+    p = _init_pdb()
     p.reset()
     if tb is None:
         return
-    while tb.tb_next is not None:
-        tb = tb.tb_next
-    p.interaction(tb.tb_frame, tb)
+    p.interaction(None, tb)
 
 
 def pm():
@@ -109,15 +155,15 @@ def pm():
 
 
 def run(statement, globals=None, locals=None):
-    Pdb(def_colors).run(statement, globals, locals)
+    _init_pdb().run(statement, globals, locals)
 
 
 def runcall(*args, **kwargs):
-    return Pdb(def_colors).runcall(*args, **kwargs)
+    return _init_pdb().runcall(*args, **kwargs)
 
 
 def runeval(expression, globals=None, locals=None):
-    return Pdb(def_colors).runeval(expression, globals, locals)
+    return _init_pdb().runeval(expression, globals, locals)
 
 
 @contextmanager
@@ -151,7 +197,7 @@ def main():
     # modified by the script being debugged. It's a bad idea when it was
     # changed by the user from the command line. There is a "restart" command
     # which allows explicit specification of command line arguments.
-    pdb = Pdb(def_colors)
+    pdb = _init_pdb()
     while 1:
         try:
             pdb._runscript(mainpyfile)
